@@ -4,6 +4,8 @@ from trianglesolver import solve, degree
 import numpy
 import math
 
+from addict import Dict
+
 
 class Distance:
 
@@ -11,19 +13,15 @@ class Distance:
         pass
 
     @staticmethod
-    def checkValidity(a, b, c):
+    def checkValidity(a: float, b: float, c: float) -> bool:
         """
-        Triangle rule checks
-        it's triangle edged as a, b, c
-        Parameters
-        ----------
-        a
-        b
-        c
-
-        Returns if is not triangle and C degree over the defined variable return False.
-        -------
-
+        Triangle rule checks it's triangle edged as a, b, c
+        Args:
+            a: edge
+            b: edge
+            c: edge
+        Returns:
+            if is not triangle and C degree over the defined variable return False.
         """
         a, b, c, A, B, C = solve(a, b, c)
         if (a + b <= c) or (a + c <= b) or (b + c <= a):
@@ -31,7 +29,35 @@ class Distance:
                 return False
         return True
 
-    def bearing(self, startLat, startLon, destLat, destLon):
+    @staticmethod
+    def bearing(**kwargs):
+        """
+        Calculate bearing(direction) from two points
+        Args:
+            **kwargs:
+             -> startLat
+             -> startLon
+             -> destLat
+             -> destLon
+        Returns:
+            Angle calculate from two point
+        """
+        """
+        
+
+        :param startLat:
+        :param startLon:
+        :param destLat:
+        :param destLon:
+        :return:
+        """
+
+        points = Dict(kwargs)
+        startLat = points.startLat
+        startLon = points.startLon
+        destLat = points.destLat
+        destLon = points.destLon
+
         phi1 = radians(startLat)
         phi2 = radians(destLat)
         cosPhi2 = cos(phi2)
@@ -41,11 +67,27 @@ class Distance:
         print(aci * TO_DEG)
         return aci * TO_DEG
 
-    def haversine(self, lon1, lat1, lon2, lat2):
+    @staticmethod
+    def haversine(**kwargs):
         """
         Calculate the great circle distance between two points
         on the earth (specified in decimal degrees)
+        Args:
+            **kwargs:
+            -> lon1
+            -> lat1
+            -> lon2
+            -> lat2
+
+        Returns:
+            distance between two points
         """
+        points = Dict(kwargs)
+        lon1 = points.lon1
+        lat1 = points.lat1
+        lon2 = points.lon2
+        lat2 = points.lat2
+
         # convert decimal degrees to radians
         lon1, lat1, lon2, lat2 = map(radians, [lon1, lat1, lon2, lat2])
 
@@ -58,44 +100,55 @@ class Distance:
         return c * RADIUS
 
     @staticmethod
-    def destinationPoint(lat1, lon1, distance, bearing):
+    def destination_point(lat, lon, distance, bearing):
+        """
+        :lat :
+        :lon :
+        :distance :
+        bearing :
+        """
+
         δ = distance / RADIUS
         θ = bearing * TO_RAD
-        φ1 = radians(lat1)
-        λ1 = radians(lon1)
+        φ1 = radians(lat)
+        λ1 = radians(lon)
         φ2 = asin(sin(φ1) * cos(δ) + cos(φ1) * sin(δ) * cos(θ))
         λ2 = λ1 + atan2(sin(θ) * sin(δ) * cos(φ1), cos(δ) - sin(φ1) * sin(φ2))
         λ2 = (λ2 + 3 * pi) % (2 * pi) - pi
 
         return {"lat": φ2 * TO_DEG, "lon": λ2 * TO_DEG}
 
-    def checkTriangleRule(self, cornerA, cornerB, cornerC) -> bool:
+    def check_triangle_rule(self, cornerA, cornerB, cornerC) -> bool:
         """
-        triangle rule
-        abs(edgeA - edgeB) < edgeC < edgeA + edgeB
-
-        Parameters
-        ----------
-        cornerA
-        cornerB
-        cornerC
-
-        Returns true or false
-        -------
+            abs(edgeA - edgeB) < edgeC < edgeA + edgeB
+        Args:
+            cornerA: edge
+            cornerB: edge
+            cornerC: edge
 
+        Returns:
+            true or false
         """
-
-        edgeAB = (self.haversine(cornerA[0], cornerA[1],
-                                 cornerB[0], cornerB[1])) * 1000
-        edgeAC = (self.haversine(cornerA[0], cornerA[1],
-                                 cornerC[0], cornerC[1])) * 1000
-        edgeBC = (self.haversine(cornerB[0], cornerB[1],
-                                 cornerC[0], cornerC[1])) * 1000
+        edgeAB = (self.haversine(lon1=cornerA[0], lat1=cornerA[1],
+                                 lon2=cornerB[0], lat2=cornerB[1])) * 1000
+        edgeAC = (self.haversine(lon1=cornerA[0], lat1=cornerA[1],
+                                 lon2=cornerC[0], lat2=cornerC[1])) * 1000
+        edgeBC = (self.haversine(lon1=cornerB[0], lat1=cornerB[1],
+                                 lon2=cornerC[0], lat2=cornerC[1])) * 1000
 
         return self.checkValidity(edgeAB, edgeAC, edgeBC)
 
     @staticmethod
-    def checkBBoxDistance(box, cfg):
+    def check_bbox_distance(box, cfg):
+        """
+        it's check bbox size from taken config params
+        Args:
+            box: detected objects bounding box
+            cfg: criterion such as bboxHeight, bboxWidth
+
+        Returns:
+
+        """
         xmin, ymin, xmax, ymax = list(map(int, box))
         # print("Y Distances = ",np.abs(ymax-ymin),
         #       "X Distances = ", np.abs(xmax-xmin))
@@ -105,7 +158,9 @@ class Distance:
             return False
         return True
 
-    def LineToXYs(self, line):  # return first and last coordinates
+    @staticmethod
+    def line_to_XYs(line):
+        # return first and last coordinates
         firstX, firstY = (line.firstPoint.X, line.firstPoint.Y)
         lastX, lastY = (line.lastPoint.X, line.lastPoint.Y)
         return [(firstX, firstY), (lastX, lastY)]
@@ -117,15 +172,14 @@ class Distance:
         The formulae used is the following:
             θ = atan2(sin(Δlong).cos(lat2),
                       cos(lat1).sin(lat2) − sin(lat1).cos(lat2).cos(Δlong))
-        :Parameters:
-          - `pointA: The tuple representing the latitude/longitude for the
-            first point. Latitude and longitude must be in decimal degrees
-          - `pointB: The tuple representing the latitude/longitude for the
-            second point. Latitude and longitude must be in decimal degrees
-        :Returns:
-          The bearing in degrees
-        :Returns Type:
-          float
+        Args:
+            pointA: The tuple representing the latitude/longitude for the
+                    first point. Latitude and longitude must be in decimal degrees
+            pointB: The tuple representing the latitude/longitude for the
+                    second point. Latitude and longitude must be in decimal degrees
+
+        Returns:
+            Angle calculate from two point
         """
         if (type(pointA) != tuple) or (type(pointB) != tuple):
             raise TypeError("Only tuples are supported as arguments")
@@ -148,3 +202,19 @@ class Distance:
         compass_bearing = (initial_bearing + 360) % 360
 
         return compass_bearing
+
+    @staticmethod
+    def distance_between_points(origin, destination):
+
+        lat1, lon1 = origin
+        lat2, lon2 = destination
+        radius = 6371  # km
+
+        dlat = math.radians(lat2 - lat1)
+        dlon = math.radians(lon2 - lon1)
+        a = math.sin(dlat / 2) * math.sin(dlat / 2) + math.cos(math.radians(lat1)) \
+            * math.cos(math.radians(lat2)) * math.sin(dlon / 2) * math.sin(dlon / 2)
+        c = 2 * math.atan2(math.sqrt(a), math.sqrt(1 - a))
+        d = radius * c
+
+        return d
